/// T-Rex : Copyright 2021 : Aldrin Mathew
///
/// AAF DEPENDABLE LICENSE - 1.0
///
/// This library is licensed under the AAF Dependable License 1.0.
/// You are allowed to use this library as a dependency in your
/// project(s) WITHOUT having to release/make available your own
/// project(s) under the same license.
///
/// Unless required by applicable law, this library is provided
/// "AS IS", WITHOUT ANY WARRANTIES OR CONDITIONS OF ANY KIND,
/// either express or implied. The author(s) of the project is
/// not liable for any harms, errors or troubles caused by using
/// this library, unless implied by law. By using this license,
/// you are acknowledging the complete terms and conditions of
/// licensing of this library as specified in AAF Dependable
/// License 1.0 available at this URL:
///
/// https://aldrinsartfactory.com/licenses/dependable/
///
/// You are NOT ALLOWED to sell, or distribute this
/// library under commercial or non-commercial purposes, other than
/// using it in your project. You are not allowed to revamp, build
/// upon, rebrand, refactor the contents of this library.
/// Your project shall contain "original work", ideally, the
/// definition of which is agreed upon by the community. Using
/// this library to achieve that is not a violation of this license.
/// You are NOT ALLOWED to use the name, branding and identity of
/// this library to identify or brand your own project. You ARE
/// however allowed to use the name and branding to pinpoint/show
/// the source of the contents/code/logic of your project. At no
/// point shall you use the identification of the Authors of this
/// library to associate them to your own projects, in a way that
/// is deceiving/misleading or gives out false information.
///

part of trex;

/// `Hook<T>` gives the developer, the power of hooks to be used in their application.
///
/// The hook will attach itself to the Hookable widget and updates the widget whenever there is a change in value. The following is the implementation of a basic hook inside a Hookable widget.
/// ```dart
/// ...
/// late final Hook<int> count;
/// void hooks() {
///   count = Hook(0, this);
/// }
/// ...
/// ```
/// You must implement the hooks function inside the Hookable widget to use hooks.
class Hook<T> {
  /// The private constructor of `Hook<T>`
  Hook(
    T value,
    this.hookableWidget,
  ) : this._value = value;

  /// The private variable holding value.
  T _value;

  /// The getter that provides the current value of variable
  T get value => this._value;

  /// The setter that updates the provided value and updates the corresponding Hookable widget for rebuild.
  set value(T newValue) {
    this._value = newValue;
    try {
      this.hookableWidget._update();
    } catch (exception) {
      print(
        'TRex Exception: Problem encountered when marking the widget element for rebuild. Native message - ' +
            exception.toString(),
      );
    }
  }

  /// The Widget that should be rebuilt when the value changes.
  Hookable hookableWidget;
}
