/// T-Rex : Copyright 2021 : Aldrin Mathew
///
/// AAF DEPENDABLE LICENSE - 1.0
///
/// This library is licensed under the AAF Dependable License 1.0.
/// You are allowed to use this library as a dependency in your
/// project(s) WITHOUT having to release/make available your own
/// project(s) under the same license.
///
/// Unless required by applicable law, this library is provided
/// "AS IS", WITHOUT ANY WARRANTIES OR CONDITIONS OF ANY KIND,
/// either express or implied. The author(s) of the project is
/// not liable for any harms, errors or troubles caused by using
/// this library, unless implied by law. By using this license,
/// you are acknowledging the complete terms and conditions of
/// licensing of this library as specified in AAF Dependable
/// License 1.0 available at this URL:
///
/// https://aldrinsartfactory.com/licenses/dependable/
///
/// You are NOT ALLOWED to sell, or distribute this
/// library under commercial or non-commercial purposes, other than
/// using it in your project. You are not allowed to revamp, build
/// upon, rebrand, refactor the contents of this library.
/// Your project shall contain "original work", ideally, the
/// definition of which is agreed upon by the community. Using
/// this library to achieve that is not a violation of this license.
/// You are NOT ALLOWED to use the name, branding and identity of
/// this library to identify or brand your own project. You ARE
/// however allowed to use the name and branding to pinpoint/show
/// the source of the contents/code/logic of your project. At no
/// point shall you use the identification of the Authors of this
/// library to associate them to your own projects, in a way that
/// is deceiving/misleading or gives out false information.
///

part of trex;

abstract class Statable<T> extends Widget {
  /// The variable that holds the StatelessElement of the corresponding widget after creation.
  late final StatableElement _statableElement;

  /// Creates a HookableElement for the widget and stores it in the private Variable.
  /// WARNING: This function must not be overriden.
  @protected
  StatableElement createElement() {
    this._statableElement = StatableElement(this);
    this.init();
    return this._statableElement;
  }

  /// The private state variable
  late final _StatableState<T> _statableState = _StatableState(null);

  T get state => this._statableState.state;

  set state(T value) {
    this._statableState.state = value;
    this._update();
  }

  /// The current context of the widget
  BuildContext get context => this._statableElement;

  /// Represents the part of the Widget that defines the UI.
  /// This is called by the Framework when the widget is created and whenever the widget needs updating.
  @protected
  Widget build(BuildContext context);

  /// Initialise the state of the widget
  @protected
  void init();

  /// This method marks this widget to be rebuilt in the next frame. This is equivalent to `setState(() {});` in a `StatefulWidget`
  void _update() {
    try {
      this._statableElement.markNeedsBuild();
    } catch (exception) {
      print(
        'TRex Exception: Problem encountered when marking the widget element for rebuild. Native message - ' +
            exception.toString(),
      );
    }
  }

  /// You can override this method to do any cleanup before the widget gets destroyed.
  ///
  /// This method will be automatically called by the framework.
  @protected
  void dispose() {}
}
